@@ -55,7 +55,7 @@ gnopiconf_client_add_dir (const gchar *path)
     
     gconf_client_add_dir (gnopernicus_client,
 			 path, 
-			 GCONF_CLIENT_PRELOAD_RECURSIVE, 
+			 GCONF_CLIENT_PRELOAD_NONE, 
 			 &error);
 
     if (error != NULL)
@@ -112,6 +112,23 @@ gnopiconf_create_schema_entry (const gchar *key,
 }
 #endif
 
+
+void
+gnopiconf_value_change (const gchar *key, 
+			const gchar *val)
+{
+    GConfValue *value;
+    GError     *error = NULL;
+    sru_assert (gnopernicus_client);
+    value = gconf_value_new (GCONF_VALUE_STRING);
+    sru_assert (value);
+    gconf_value_set_string (value, val);
+    gconf_client_set (gnopernicus_client, key, value, &error);
+    gconf_value_free (value);
+    if (error)
+	fprintf (stderr, "Error in set");
+}
+
 static gboolean
 gnopiconf_check_type (const gchar* key, 
 		      GConfValue* val, 
@@ -189,6 +206,9 @@ gnopiconf_set_string (const gchar *val,
     gnopiconf_create_schema_entry (key, "string", val, NULL);
 #endif
     
+    if (!val)
+	val = "";
+	
     ret = gconf_client_set_string (gnopernicus_client, key, val, &error);
 
     if (error != NULL)
@@ -198,7 +218,7 @@ gnopiconf_set_string (const gchar *val,
 	g_error_free (error);
 	error = NULL;
     }
-        
+gconf_client_suggest_sync (gnopernicus_client, NULL);        
     return ret;
 }
 
