@@ -37,7 +37,7 @@ PresentationStruct default_presentations[] =
 		{"braille",	"check-box",	"present",	N_("%role%%name%%(shortcut)%%?state:checked?\"(C)\":\"( )\"?%%?state:enabled?\"\":\"(U)\"?%")},
 		{"braille",	"check-menu-item","generic",	N_("%role%%name%%(accelerator)%%(shortcut)%%?state:checked?\"(C)\":\"( )\"?%%?state:enabled?\"\":\"(U)\"?%")},
 		{"braille",	"combo-box",	"generic",	N_("%role%%name%%text:line%")},
-		{"braille",	"label",	"generic",	N_("%role%%text:line%")},
+		{"braille",	"label",	"generic",	N_("%role%%name%")},
 		{"braille",	"link",		"generic",	N_("%role%%text:line%")},
 		{"braille",	"list",		"generic",	N_("%role%%name%%childcount%%1childcount?\"item\":\"items\"1%")},
 		{"braille",	"list-item",	"generic",	N_("%role%%name%%(shortcut)%")},		
@@ -74,7 +74,7 @@ PresentationStruct default_presentations[] =
 	    	{"speech",	"check-menu-item","object:state-changed:checked",	N_("%?state:checked?\"checked\":\"unchecked\"<voice=state>?%")},
 		{"speech",	"combo-box",	"focus:",	N_("%name<voice=name>%%role<voice=role>%%text:line<voice=text>%")},
 	    	{"speech",	"combo-box",	"object:property-changed:accessible-content",N_("%text:line<voice=text>%")},
-		{"speech", 	"label", 	"generic", 	N_("%text:line<voice=name>%%role<voice=role>%")},
+		{"speech", 	"label", 	"generic", 	N_("%name<voice=name>%%role<voice=role>%")},
 		{"speech", 	"link", 	"generic", 	N_("%text:line<voice=name>%%role<voice=role>%")},
 		{"speech",	"list",		"generic",	N_("%name<voice=name>%%role<voice=role>%%childcount<voice=childcount>%%1childcount?\"item\":\"items\"<voice=system>1%")},
 		{"speech", 	"list-item", 	"generic", 	N_("%name<voice=name>%%role<voice=role>%")},
