@@ -53,6 +53,7 @@ typedef struct
     gint                          	voice_idx;
     GNOME_Speech_Speaker          	speaker;
     BonoboObject        		*callback;
+    gboolean				cb_support;
     /*parameters MIN and MAX*/
     GSMinMaxType			rate;
     GSMinMaxType			pitch;
@@ -157,7 +158,6 @@ gs_speaker_free (GSSpeaker *gss)
 	fprintf (stderr,"\ngs_speaker_free : voice_idx = %d",
 			    gss->voice_idx);
 #endif
-	if (gss->callback != CORBA_OBJECT_NIL)
 /*
 	    GNOME_Speech_SpeechCallback_unref (gss->callback, NULL);
 */
@@ -231,22 +231,25 @@ gs_speaker_procent_to_units (GSSpeaker	*speaker,
     
 }
 
-void
-gs_speaker_to_gconf (gint	voice_n, 
-		     char 	*voice_name)
+gchar*
+gs_normilize_driver_name (gchar *driver_name)
 {
-    gchar	*voice_key; 
-
-     /* !!! TBI !!! voice_#_language, voice_#_gender*/
-    voice_key 	= g_strdup_printf ("voice_%d_name", 
-				    voice_n);
-	    
-    srconf_set_data (voice_key, 
-		     CFGT_STRING, 
-		     voice_name, 
-		     SPEECH_PARAMETER_SECTION);
+    gchar *new_driver_name;
     
-    g_free (voice_key);
+    new_driver_name = g_strdelimit (driver_name, "|> <", '_');
+    
+    return new_driver_name;
+}
+void
+gs_speaker_to_gconf (GSList *driver_voice_list, 
+		    const gchar *driver_name)
+{
+    gchar *name = gs_normilize_driver_name ((gchar*)driver_name);
+    srconf_set_data (name, 
+		     CFGT_LIST, 
+		     driver_voice_list, 
+		     SPEECH_DRIVERS_SECTION);
+    g_free (name);
 }
 /*_____________________________</GS SPEAKER >_________________________________*/
 
@@ -275,7 +278,9 @@ gs_speakers_flush (gpointer 	key,
 void
 gs_speakers_terminate (void)
 {
-
+	GSList *drivers = NULL;
+	GSList *tmp = NULL;
+	
 	if (!gs_speakers_hash_table) 
 	    return;
 	
@@ -284,6 +289,23 @@ gs_speakers_terminate (void)
 			      NULL);
 	g_hash_table_destroy (gs_speakers_hash_table);
 	gs_speakers_hash_table = NULL;
+
+	srconf_get_data_with_default(SPEECH_ENGINE_DRIVERS, 
+				    CFGT_LIST, &drivers,
+				    NULL, 
+				    SPEECH_DRIVERS_SECTION);
+	for (tmp = drivers ; tmp ; tmp = tmp->next)
+	{
+	    srconf_unset_key ((gchar*)tmp->data, 
+		             SPEECH_DRIVERS_SECTION);
+	    g_free (tmp->data);
+	}
+	g_slist_free (drivers);
+	
+	srconf_unset_key (SPEECH_ENGINE_DRIVERS, 
+		    	  SPEECH_DRIVERS_SECTION);
+	srconf_unset_key (SPEECH_VOICE_COUNT, 
+		    	  SPEECH_DRIVERS_SECTION);
 }
 
 GSSpeaker *
@@ -426,8 +448,18 @@ gs_speakers_select_speaker (gchar *voice_name)
 		GNOME_Speech_Speaker_registerSpeechCallback (gss->speaker,
 							bonobo_object_corba_objref (bo),
 							&ev);
-		check_return_value (&ev, __LINE__);							
-		
+		if ( (&ev) && (&ev)->_major != CORBA_NO_EXCEPTION)
+		{
+		    sru_message ("Callbacks are NOT supported by all drivers");
+		    
+		    bonobo_object_unref (gss->callback);
+		    
+		    gss->callback = NULL;
+		    gss->cb_support = FALSE;
+		}
+		else
+		    gss->cb_support = TRUE;
+		    
 		CORBA_free (parameters);
 	    }
 	    CORBA_free (voices);
@@ -437,13 +469,22 @@ gs_speakers_select_speaker (gchar *voice_name)
     return gss;
 }
 
+void
+gs_driver_to_gconf (GSList *speech_driver_list)
+{
+    srconf_set_data (SPEECH_ENGINE_DRIVERS, 
+		     CFGT_LIST, 
+		     speech_driver_list, 
+		     SPEECH_DRIVERS_SECTION);
+}
+
 void
 gs_speakers_to_gconf (gint 	voice_cnt)
 {
-    srconf_set_data ("voice_count", 
+    srconf_set_data (SPEECH_VOICE_COUNT, 
 		     CFGT_INT, 
 		     &voice_cnt, 
-		     SPEECH_PARAMETER_SECTION);
+		     SPEECH_DRIVERS_SECTION);
 }
 /*_______________________</GS SPEAKERS CONTAINER >____________________________*/
 
@@ -456,7 +497,25 @@ gs_speak (SRSVoice 	*voice,
     /* sel current speaker for voice->TTSVoiceName */
     gss = gs_speakers_select_speaker (voice->tts_voice_name);
     if (gss)
+    {
 	current_speaker = gss->speaker;
+    
+	if (!gss->cb_support )
+	{
+	    if (voice->cb_support == -1)
+    		sru_warning ("Voice %s is NOT supporting speech markers",
+			voice->tts_voice_name);
+	    voice->cb_support = 0;
+	}
+	else
+	{
+/*	
+	    sru_message ("Voice %s is supporting speech markers",
+			voice->tts_voice_name);
+*/
+	    voice->cb_support = 1;
+	}
+    }
     if (current_speaker)
     {
 	CORBA_Environment 	ev;
@@ -627,6 +686,9 @@ gs_init (TTSEngine *engine)
     Bonobo_ServerInfoList 	*servers;
     GSSpeaker			*gss;
 
+    GSList			*speech_driver_list = NULL;
+    GSList			*driver_voice_list  = NULL;
+    GSList			*tmp;
 /*FIXME*/
     /*create internal hash table*/
     gs_speakers_init();
@@ -659,6 +721,9 @@ gs_init (TTSEngine *engine)
 								    &ev);
 	check_return_value (&ev, __LINE__);
 
+	speech_driver_list = g_slist_append (speech_driver_list, 
+					     g_strdup (gs_normilize_driver_name (driver_name)));
+
 	/* create a list of available voices in gconf */
 	voices = GNOME_Speech_SynthesisDriver_getAllVoices (driver, 
 							    &ev);
@@ -680,7 +745,10 @@ gs_init (TTSEngine *engine)
 					    voice_n, 
 					    voices->_buffer[voice_n].name, 
 					    driver_name );
-	    gs_speaker_to_gconf (voice_cnt++, voice_name);
+	    driver_voice_list = g_slist_append (driver_voice_list,
+						g_strdup (voice_name));
+
+	    voice_cnt++;
     	    gs_speakers_add_speaker (gss, voice_name);
 
     	    /* select a default voice */
@@ -695,11 +763,26 @@ gs_init (TTSEngine *engine)
     	    g_free (voice_name);
 	}/*end of voices*/
 	
+        gs_speaker_to_gconf (driver_voice_list, driver_name);
+	for (tmp = driver_voice_list; tmp ; tmp = tmp->next)
+	    g_free (tmp->data);
+	g_slist_free (driver_voice_list);
+	
+	driver_voice_list = NULL;
+
 	CORBA_free (voices);
 	CORBA_free (driver_name);	
 	CORBA_Object_release (driver, NULL);
     } /* end of for server */
+    
     gs_speakers_to_gconf (voice_cnt);
+    gs_driver_to_gconf (speech_driver_list);    
+    
+    for (tmp = speech_driver_list; tmp ; tmp = tmp->next)
+	g_free (tmp->data);
+	
+    g_slist_free (speech_driver_list);
+
     CORBA_free (servers);
 
     /* select a default speaker */
