@@ -697,7 +697,7 @@ src_braille_init ()
     if (!brl_xml_init(src_braille_device, src_braille_port, brl_xml_input_proc)) /* take Braille display from Gnopi... */
     {
 	brlval = FALSE;
-	fprintf(stderr, "** BRAILLE initialization failed for %s on port %d",
+	fprintf(stderr, "\n** BRAILLE initialization failed for %s on port %d",
 					src_braille_device, src_braille_port);
 	fprintf (stderr, "\n   To change one or more settings use gnopernicus UI\n");
     }
