@@ -40,7 +40,6 @@ extern gboolean src_use_magnifier;
 extern gboolean src_use_speech;
 extern gboolean src_use_braille;
 extern gboolean src_use_braille_monitor;
-extern gboolean panning;
 extern gboolean src_enable_format_string;
 
 gboolean  src_button_left_pressed   = FALSE;
@@ -696,7 +695,7 @@ src_mag_panning_on_off ()
 {
     if (src_use_magnifier) 
     {
-/*	gboolean panning;*/
+	gboolean panning;
 	gboolean default_panning = DEFAULT_MAGNIFIER_PANNING;
 	if (!srconf_get_data_with_default (MAGNIFIER_PANNING, 
 					   CFGT_BOOL, 
@@ -705,11 +704,9 @@ src_mag_panning_on_off ()
 					   MAGNIFIER_CONFIG_PATH)
 	    ) 
 	    return FALSE;
+
 	panning = !panning;
-	if (!panning) 
-	{
-	    src_magnifier_stop_panning (0);
-	}	    
+
 	if (!srconf_set_data (MAGNIFIER_PANNING,
 			      CFGT_BOOL, 
 			      &panning, 
