@@ -1,4 +1,4 @@
-/* srspc.c
+ /* srspc.c
  *
  * Copyright 2001, 2002 Sun Microsystems, Inc.,
  * Copyright 2001, 2002 BAUM Retec, A.G.
@@ -50,6 +50,7 @@ SRCSpeechDictionaryType    src_speech_dictionary_type    = SRC_SPEECH_DICTIONARY
 static SRCSpeechSpellingMode      src_speech_spelling_mode      = SRC_SPEECH_SPELL_NORMAL;
 static SRCSpeechCountMode     	  src_speech_count_mode         = SRC_SPEECH_COUNT_NONE;
 static gchar 	**src_speech_engine_voices = NULL;
+static gchar 	**src_speech_engine_drivers= NULL;
 static gchar 	**src_speech_voices        = NULL;
 static gboolean src_speech_initialized     = FALSE;
 
@@ -57,36 +58,87 @@ static GHashTable *src_dictionary_hash 	= NULL;
 
 
 static gchar**
-src_speech_get_engine_voices ()
+src_speech_get_engine_drivers ()
 {
-    gint index, index2;
-    gint count = -1;
     gchar **rv;
-
+    gint  count;
+    gint  index;
+    GSList *drivers = NULL;
+    GSList *tmp     = NULL;
+    
     sru_assert (src_speech_initialized);
 
+    srconf_get_data_with_default (SPEECH_ENGINE_DRIVERS, CFGT_LIST, 
+				&drivers, NULL, SPEECH_DRIVERS_SECTION);
+
+    if (!drivers)
+	return NULL;
+	
+    count = g_slist_length (drivers);
+    
+    rv = g_new (gchar*, count + 1);
+        
+    index = 0;
+    
+    for (tmp = drivers; tmp ; tmp = tmp->next)
+    {
+	if (tmp->data)
+	{
+	    rv[index] = g_strdup (tmp->data);
+	    /*fprintf (stderr, "\nENGINE DRIVER[%d]=%s", index, (gchar*)tmp->data);*/
+	    index++;
+	}
+	g_free (tmp->data);
+    }
+    
+    rv[index] = NULL;
+    
+    g_slist_free (drivers);
+    
+    return rv;
+}
+
+static gchar**
+src_speech_get_engine_voices ()
+{
+    gint index2;
+    gint iter;
+    gint count = -1;
+    gchar **rv;
+    GSList *voices  = NULL;
+    GSList *tmp2    = NULL;
+
+    sru_assert (src_speech_initialized);
+    sru_assert (src_speech_engine_drivers);
+    
+
     srconf_get_data_with_default (SPEECH_VOICE_COUNT, CFGT_INT, 
-				&count, &count, SPEECH_PARAMETER_SECTION);
+				&count, &count, SPEECH_DRIVERS_SECTION);
 
     if (count <= 0)
 	return NULL;
-    
+
     rv = g_new (gchar*, count + 1);
+    
     index2 = 0;
-    for (index = 0; index < count; index++)
+    iter = 0;
+
+    for (iter = 0; src_speech_engine_drivers[iter] ; iter++)
     {
-	gchar *path, *tmp;
-	path = g_strdup_printf ("voice_%d_name", index);
-	srconf_get_data_with_default (path, CFGT_STRING, &tmp, NULL,
-					    SPEECH_PARAMETER_SECTION);
-	if (tmp && tmp[0])
+	srconf_get_data_with_default (src_speech_engine_drivers[iter], CFGT_LIST, 
+				      &voices, NULL, SPEECH_DRIVERS_SECTION);
+	for (tmp2 = voices; tmp2 ; tmp2 = tmp2->next)
 	{
-	    rv[index2] = g_strdup (tmp);
-/*	     fprintf (stderr, "\nENGINE VOICE[%d]=%s", index2, rv[index2]);*/
-	    index2++;
+	    if (tmp2->data && ((gchar*)tmp2->data)[0])
+	    {
+		rv[index2] = g_strdup (tmp2->data);
+/*		fprintf (stderr, "\nENGINE VOICE[%d]=%s", index2, rv[index2]);*/
+		index2++;
+	    }
+	    g_free (tmp2->data);
 	}
-	g_free (tmp);
-	g_free (path);
+	g_slist_free (voices);
+	voices = NULL;
     }
     rv[index2] = NULL;
 
@@ -95,14 +147,26 @@ src_speech_get_engine_voices ()
 
 
 static gboolean
-src_speech_valid_engine_voice (gchar *voice)
+src_speech_valid_engine_voice ( gchar *driver,
+				gchar *voice)
 {
     gint i;
+    gboolean valid_driver = FALSE;
 
     sru_assert (src_speech_initialized);
     sru_assert (src_speech_engine_voices);
+    sru_assert (src_speech_engine_drivers);
     sru_assert (voice);
 
+    for (i = 0; src_speech_engine_drivers[i]; i++)
+    {
+	if (strcmp (driver, src_speech_engine_drivers[i]) == 0)
+	    valid_driver = TRUE;
+    }
+
+    if (!valid_driver)
+	return FALSE;
+	
     for (i = 0; src_speech_engine_voices[i]; i++)
     {
 	if (strcmp (voice, src_speech_engine_voices[i]) == 0)
@@ -185,6 +249,7 @@ src_speech_valid_voice (gchar *voice)
 
 static gboolean
 src_speech_get_voice_parameters (gchar *voice,
+				 gchar **driver, 
 				 gchar **name,
 				 gchar **volume,
 				 gchar **rate,
@@ -201,29 +266,31 @@ src_speech_get_voice_parameters (gchar *voice,
     if (voices)
     {
 	GSList *crt = voices;
-
+	
+	if (driver)
+	    src_speech_set_string_from_list_element (crt, driver);
+	if (crt)
+	    crt = crt->next;
 	if (name)
 	    src_speech_set_string_from_list_element (crt, name);
-
 	if (crt)
 	    crt = crt->next;
 	if (volume)
 	    src_speech_set_string_from_list_element (crt, volume);
-
 	if (crt)
 	    crt = crt->next;
 	if (rate)
 	    src_speech_set_string_from_list_element (crt, rate);
-
 	if (crt)
 	    crt = crt->next;
 	if (pitch)
 	    src_speech_set_string_from_list_element (crt, pitch);
-
 	srconf_free_slist (voices);
     }
     else
     {
+	if (driver)
+	    *driver = NULL;
 	if (name)
 	    *name = NULL;
 	if (volume)
@@ -245,19 +312,25 @@ src_speech_get_voice_parameters (gchar *voice,
 
 static gboolean
 src_speech_set_voice_parameters (gchar *voice,
+				 gchar *driver_,
 				 gchar *name_,
 				 gchar *volume_,
 				 gchar *rate_,
 				 gchar *pitch_)
 {
-    gchar *name, *volume, *rate, *pitch;
+    gchar *driver, *name, *volume, *rate, *pitch;
     gboolean rv = FALSE;
 
     sru_assert (src_speech_initialized);
     sru_assert (voice && voice[0]);
 
-    src_speech_get_voice_parameters (voice, &name, &volume, &rate, &pitch);
+    src_speech_get_voice_parameters (voice, &driver, &name, &volume, &rate, &pitch);
 
+    if (driver_)
+    {
+	g_free (driver);
+	driver = g_strdup (driver_);
+    }
     if (name_)
     {
 	g_free (name);
@@ -278,14 +351,15 @@ src_speech_set_voice_parameters (gchar *voice,
 	g_free (pitch);
 	pitch = g_strdup (pitch_);
     }
-    if (! (name && name[0] && volume && volume[0] && rate && rate[0] 
-		&& pitch && pitch[0]))
+    if (! (driver && driver[0] && name && name[0] && volume && volume[0] && 
+	    rate && rate[0] && pitch && pitch[0]))
     {
-	sru_warning (_("incorect parameters for voice \"%s\""), voice); 
+	sru_warning (_("incorect parameters for voice \"%s%s\""), driver, voice); 
     }
     else
     {
         GSList *list = NULL;
+	list = g_slist_append (list, driver);
 	list = g_slist_append (list, name);
 	list = g_slist_append (list, volume);
 	list = g_slist_append (list, rate);
@@ -331,6 +405,7 @@ src_speech_send (gchar *spcoutput)
 
 static gboolean
 src_speech_create_voice (gchar *voice,
+			 gchar *driver_,
 			 gchar *name_,
 			 gchar *volume_,
 			 gchar *rate_,
@@ -345,9 +420,9 @@ src_speech_create_voice (gchar *voice,
     sru_assert (voice);
 
     name = volume = rate = pitch = NULL;
-    if (name_)
+    if (name_ && driver_)
     {
-	sru_assert (src_speech_valid_engine_voice (name_));
+	sru_assert (src_speech_valid_engine_voice (driver_, name_));
 	name = g_strconcat (" TTSVoice=\"", name_, "\"", NULL);
     }
     if (rate_)
@@ -376,7 +451,12 @@ src_speech_create_voice (gchar *voice,
     if (!rv)
 	sru_warning (_("could not create/modify voice \"%s\""), voice);
     g_free (voice_);
-
+    
+    g_free (name);
+    g_free (rate);
+    g_free (pitch);
+    g_free (volume);
+    
     return rv;
 }
 
@@ -384,24 +464,28 @@ src_speech_create_voice (gchar *voice,
 gboolean
 src_speech_init_voice (gchar *voice)
 {
-    gchar *name, *volume, *rate, *pitch;
+    gchar *driver, *name, *volume, *rate, *pitch;
     gboolean rv = FALSE;
 
     sru_assert (src_speech_initialized);
     sru_assert (voice && voice[0]);
     
-    name = volume = rate = pitch = NULL;
-    if (src_speech_get_voice_parameters (voice, &name, &volume, &rate, &pitch))
+    driver = name = volume = rate = pitch = NULL;
+    if (src_speech_get_voice_parameters (voice, &driver, &name, &volume, &rate, &pitch))
     {
-	if (!name || !src_speech_valid_engine_voice (name))
+	if (!name || !driver ||
+	    !src_speech_valid_engine_voice (driver, name))
 	{
 	    g_free (name);
+	    g_free (driver);
 	    name = g_strdup (src_speech_engine_voices[0]);
-	    src_speech_set_voice_parameters (voice, name, volume, rate, pitch);
+	    driver = g_strdup (src_speech_engine_drivers[0]);
+	    src_speech_set_voice_parameters (voice, driver, name, volume, rate, pitch);
 	}
-	rv = src_speech_create_voice (voice, name, volume, rate, pitch);
+	rv = src_speech_create_voice (voice, driver, name, volume, rate, pitch);
     }
 
+    g_free (driver);
     g_free (name);
     g_free (volume);
     g_free (rate);
@@ -501,7 +585,7 @@ src_speech_remove_voice (gchar *voice)
 gboolean
 src_speech_test_voice (gchar *voice)
 {
-    gchar *name, *volume, *rate, *pitch;
+    gchar *driver, *name, *volume, *rate, *pitch;
     gboolean rv = FALSE;
 
     sru_assert (src_speech_initialized);
@@ -512,10 +596,11 @@ src_speech_test_voice (gchar *voice)
 	return FALSE;
 
     name = volume = rate = pitch = NULL;
-    if (src_speech_get_voice_parameters (voice, &name, &volume, &rate, &pitch))
+    if (src_speech_get_voice_parameters (voice, &driver, &name, &volume, &rate, &pitch))
     {
-	if (name && src_speech_valid_engine_voice (name))
-	    rv = src_speech_create_voice ("test", name, volume, rate, pitch);
+	if (name && driver && 
+	    src_speech_valid_engine_voice (driver, name))
+	    rv = src_speech_create_voice ("test", driver, name, volume, rate, pitch);
     }
 
     if (rv)
@@ -529,6 +614,7 @@ src_speech_test_voice (gchar *voice)
 	sru_warning (_("could not test voice \"%s\""), voice);
 
     src_speech_remove_voice ("test");
+    g_free (driver);
     g_free (name);
     g_free (volume);
     g_free (rate);
@@ -1063,17 +1149,18 @@ static gboolean
 src_speech_init_default_voice ()
 {
     gboolean rv = FALSE;
-    gchar *name, *volume, *rate, *pitch;
+    gchar *driver, *name, *volume, *rate, *pitch;
  
     sru_assert (src_speech_initialized);
     sru_assert (src_speech_engine_voices);
    
+    driver = g_strdup (src_speech_engine_drivers[0]);
     name   = g_strdup (src_speech_engine_voices[0]);
     volume = g_strdup_printf ("%d", DEFAULT_SPEECH_VOLUME);
     rate   = g_strdup_printf ("%d", DEFAULT_SPEECH_RATE);
     pitch  = g_strdup_printf ("%d", DEFAULT_SPEECH_PITCH);
-    rv = src_speech_set_voice_parameters ("default", name, volume, rate, pitch);
-
+    rv = src_speech_set_voice_parameters ("default", driver, name, volume, rate, pitch);
+    g_free (driver);
     g_free (name);
     g_free (rate);
     g_free (volume);
@@ -1165,10 +1252,11 @@ src_speech_init ()
 
     if (src_speech_initialized)
     {
-	src_speech_engine_voices = src_speech_get_engine_voices ();
-	src_speech_voices = src_speech_get_voices ();
+	src_speech_engine_drivers= src_speech_get_engine_drivers ();
+	src_speech_engine_voices = src_speech_get_engine_voices  ();
+	src_speech_voices 	 = src_speech_get_voices ();
     }
-
+    
     if (src_speech_initialized && src_speech_engine_voices && src_speech_voices)
     {
 	src_speech_initialized = src_speech_init_default_voice () &&
@@ -1187,6 +1275,7 @@ src_speech_init ()
 	src_speech_set_spelling_mode (SRC_SPEECH_SPELL_NORMAL); /*FIXME: read from gconf????*/
 	src_speech_load_defaults ();
     }
+
     srconf_unset_key (SRCORE_SPEECH_SENSITIVE, SRCORE_PATH);/*FIXME:why this line */
     SET_SRCORE_CONFIG_DATA (SRCORE_SPEECH_SENSITIVE, CFGT_BOOL, 
 				&src_speech_initialized);/*FIXME: must be in speech gconf directory */
@@ -1205,7 +1294,6 @@ src_speech_init ()
 	g_strfreev (src_speech_engine_voices);
     }
 
-    
     return src_speech_initialized;
 }
 
@@ -1221,6 +1309,7 @@ src_speech_terminate ()
     src_speech_initialized = FALSE;    
     g_strfreev (src_speech_voices);
     g_strfreev (src_speech_engine_voices);
+    g_strfreev (src_speech_engine_drivers);
 }
 
 #define src_speech_is_separator(x) (( x == '.' || x == ';' || x == '?' || x == ',' || x == '!' ) ? TRUE : FALSE)
@@ -1507,7 +1596,7 @@ src_speech_modify_pitch (SRCModifyMode mode)
 	
 	    if (mode != SRC_MODIF_DEFAULT)
 	    {
-		if (src_speech_get_voice_parameters (voices[i], NULL, NULL, NULL, &pitch_))
+		if (src_speech_get_voice_parameters (voices[i], NULL, NULL, NULL, NULL, &pitch_))
 		{
 		    pitch = atoi (pitch_);
 		    g_free (pitch_);
@@ -1519,8 +1608,8 @@ src_speech_modify_pitch (SRCModifyMode mode)
 		pitch = DEFAULT_SPEECH_PITCH;
 	    pitch = MIN (MAX (pitch, MIN_SPEECH_PITCH), MAX_SPEECH_PITCH);
 	    pitch_ = g_strdup_printf ("%d", pitch);
-	    if (src_speech_set_voice_parameters (voices[i], NULL, NULL, NULL, pitch_))
-		src_speech_create_voice (voices[i], NULL, NULL, NULL, pitch_);
+	    if (src_speech_set_voice_parameters (voices[i], NULL, NULL, NULL, NULL, pitch_))
+		src_speech_create_voice (voices[i], NULL, NULL, NULL, NULL, pitch_);
 	    g_free (pitch_);	    
 	}
     }
@@ -1557,7 +1646,7 @@ src_speech_modify_rate (SRCModifyMode mode)
 	
 	    if (mode != SRC_MODIF_DEFAULT)
 	    {
-		if (src_speech_get_voice_parameters (voices[i], NULL, NULL, &rate_, NULL))
+		if (src_speech_get_voice_parameters (voices[i], NULL, NULL, NULL, &rate_, NULL))
 		{
 		    rate = atoi (rate_);
 		    g_free (rate_);
@@ -1569,8 +1658,8 @@ src_speech_modify_rate (SRCModifyMode mode)
 		rate = DEFAULT_SPEECH_RATE;
 	    rate = MIN (MAX (rate, MIN_SPEECH_RATE), MAX_SPEECH_RATE);
 	    rate_ = g_strdup_printf ("%d", rate);
-	    if (src_speech_set_voice_parameters (voices[i], NULL, NULL, rate_, NULL))
-		src_speech_create_voice (voices[i], NULL, NULL, rate_, NULL);
+	    if (src_speech_set_voice_parameters (voices[i], NULL, NULL, NULL, rate_, NULL))
+		src_speech_create_voice (voices[i], NULL, NULL, NULL, rate_, NULL);
 	    g_free (rate_);	    
 	}
     }
@@ -1608,7 +1697,7 @@ src_speech_modify_volume (SRCModifyMode mode)
 	
 	    if (mode != SRC_MODIF_DEFAULT)
 	    {
-		if (src_speech_get_voice_parameters (voices[i], NULL, &volume_, NULL, NULL))
+		if (src_speech_get_voice_parameters (voices[i], NULL, NULL, &volume_, NULL, NULL))
 		{
 		    volume = atoi (volume_);
 		    g_free (volume_);
@@ -1620,8 +1709,8 @@ src_speech_modify_volume (SRCModifyMode mode)
 		volume = DEFAULT_SPEECH_VOLUME;
 	    volume = MIN (MAX (volume, MIN_SPEECH_VOLUME), MAX_SPEECH_VOLUME);
 	    volume_ = g_strdup_printf ("%d", volume);
-	    if (src_speech_set_voice_parameters (voices[i], NULL, volume_, NULL, NULL))
-		src_speech_create_voice (voices[i], NULL, volume_, NULL, NULL);
+	    if (src_speech_set_voice_parameters (voices[i], NULL, NULL, volume_, NULL, NULL))
+		src_speech_create_voice (voices[i], NULL, NULL, volume_, NULL, NULL);
 	    g_free (volume_);	    
 	}
     }
@@ -1629,6 +1718,24 @@ src_speech_modify_volume (SRCModifyMode mode)
     return TRUE;
 }
 
+gboolean
+src_speech_proces_voice_config_changed (SRConfigStructure *config)
+{
+    gboolean rv = TRUE;
+    
+    sru_assert (src_speech_initialized);
+    sru_assert (config && config->key);
+    sru_assert (config->type == CFGT_LIST);
+    sru_assert (config->newvalue);
+	
+    if (!strcmp (config->key, SPEECH_GNOPERNICUS_SPEAKERS))
+	    return rv;
+	
+    rv = src_speech_add_voice (config->key) &&
+	    src_speech_init_voice (config->key);
+    
+    return rv;
+}
 
 gboolean
 src_speech_proces_config_changed (SRConfigStructure *config)
@@ -1636,7 +1743,7 @@ src_speech_proces_config_changed (SRConfigStructure *config)
     gboolean rv = TRUE;
     sru_assert (src_speech_initialized);
     sru_assert (config && config->key);
-    
+
     if (strcmp (config->key, SPEECH_COUNT) == 0)
     {
 	sru_assert (config->type == CFGT_STRING);
@@ -1655,13 +1762,6 @@ src_speech_proces_config_changed (SRConfigStructure *config)
 	sru_assert (config->newvalue);
 	rv = src_speech_test_voice (config->newvalue);
     }
-    else if (strcmp (config->key, SPEECH_VOICE_CHANGES) == 0)
-    {
-	sru_assert (config->type == CFGT_STRING);
-	sru_assert (config->newvalue);
-	rv = src_speech_add_voice (config->newvalue) &&
-		    src_speech_init_voice (config->newvalue);
-    }
     else if (strcmp (config->key, SPEECH_VOICE_REMOVED) == 0)
     {
 	sru_assert (config->type == CFGT_STRING);
