@@ -71,12 +71,123 @@ static gchar	  *srl_last_deleted_text = NULL;
 static SREvent    *srl_last_event = NULL;
 static unsigned long srl_last_flags = 0;
 
-
 #ifdef SRL_DEBUG
 GList *srl_obj_list = NULL;
 GList *srl_str_list = NULL;
 #endif
 
+#define SRL_LOG_AT_SPI		1
+#define SRL_LOG_GNOPERNICUS	2
+#define SRL_LOG_IMPORTANT	4
+#define SRL_LOG_TERMINAL	8
+#define SRL_LOG_REENTRANCY	16
+
+static gint srl_log_flags = 0;
+
+static gchar*
+srl_get_toolkit (Accessible *acc)
+{
+    Accessible *parent;
+    gchar *rv = NULL;
+    
+    srl_return_val_if_fail (acc, NULL);
+    
+    parent = acc;
+    Accessible_ref (parent);
+    
+    while (parent && !Accessible_isApplication (parent))
+    {
+	acc = parent;
+	parent = Accessible_getParent (parent);
+	Accessible_unref (acc);
+    }
+    if (parent)
+    {
+	AccessibleApplication *app;
+	app = Accessible_getApplication (parent);
+	if (app)
+	{
+	    rv = AccessibleApplication_getToolkitName (app);
+	    AccessibleApplication_unref (app);
+	}
+	Accessible_unref (parent);
+    }
+    return rv;
+}
+
+
+static void
+srl_log_at_spi (const AccessibleEvent *event)
+{
+    gchar *name, *role, *toolkit;
+    if (!(srl_log_flags & SRL_LOG_AT_SPI))
+	return;
+    if (!(srl_log_flags & SRL_LOG_TERMINAL) && 
+	    Accessible_getRole (event->source) == SPI_ROLE_TERMINAL)
+	return;
+
+    name = Accessible_getName (event->source),
+    role = Accessible_getRoleName (event->source);
+    toolkit = srl_get_toolkit (event->source);
+    fprintf (stderr, "\nAT:%xp----\"%s\" "
+			"for %xp \"%s\" role \"%s\" "
+			"from \"%s\" with details %ld and %ld", 
+		(unsigned int)event, event->type,
+		(unsigned int)event->source, name ? name : "", role ? role : "",
+		toolkit ? toolkit : "", event->detail1, event->detail2);
+    SPI_freeString (name);
+    SPI_freeString (role);
+    SPI_freeString (toolkit);
+}
+
+static void
+srl_log_gnopernicus (SREvent *event)
+{
+    SRObject *obj;
+    gchar *name, *role, *type;
+    if (!(srl_log_flags & SRL_LOG_GNOPERNICUS))
+	return;
+
+    sre_get_event_data (event, (void**)&obj);
+    sro_get_name (obj, &name, SR_INDEX_CONTAINER);
+    sro_get_role_name (obj, &role, SR_INDEX_CONTAINER);
+    sro_get_reason (obj, &type);
+    fprintf (stderr, "\nGN:%xp--\"%s\" "
+			"for %xp \"%s\" role \"%s\" ", 
+		(unsigned int)event, type ? type : "",
+		(unsigned int)obj, name ? name : "", role ? role : "");
+    SR_freeString (name);
+    SR_freeString (role);
+    SR_freeString (type);
+}
+
+static void
+srl_log_gnopernicus_real (SREvent *event)
+{
+    if (!(srl_log_flags & SRL_LOG_GNOPERNICUS))
+	return;
+}
+
+static void
+srl_log_important (SRLEvent *event, SREvent *ev)
+{
+    if (!(srl_log_flags & SRL_LOG_AT_SPI))
+	return;
+}
+
+static void
+srl_log_reentrancy (const AccessibleEvent *event)
+{
+    if (!(srl_log_flags & SRL_LOG_REENTRANCY))
+	return;
+    if (!(srl_log_flags & SRL_LOG_TERMINAL) && 
+	    Accessible_getRole (event->source) == SPI_ROLE_TERMINAL)
+	return;
+
+    fprintf (stderr, "\nQU:%xp will be really added to gnopernicus queue",
+    		(unsigned int)event);
+}
+
 /**
  * srl_init:
  *
@@ -88,6 +199,7 @@ gboolean
 srl_init ()
 {
     gboolean rv = TRUE;
+    const gchar *log;
     int  i;
     
     srl_assert (srl_initialized == 0);
@@ -164,6 +276,20 @@ srl_init ()
     srl_obj_list = NULL;
     srl_str_list = NULL;
 #endif
+    log = g_getenv ("GNOPERNICUS_LOG");
+    if (!log)
+	log = "";
+    srl_log_flags = 0;
+    if (strstr (log, "at-spi"))
+	srl_log_flags |= SRL_LOG_AT_SPI;
+    if (strstr (log, "gnopernicus"))
+	srl_log_flags |= SRL_LOG_GNOPERNICUS;
+    if (strstr (log, "important"))
+	srl_log_flags |= SRL_LOG_IMPORTANT;
+    if (strstr (log, "terminal"))
+	srl_log_flags |= SRL_LOG_TERMINAL;
+    if (strstr (log, "reentrancy"))
+	srl_log_flags |= SRL_LOG_REENTRANCY;
     return rv;
 }
 
@@ -323,6 +449,8 @@ notify_clients_ (gpointer data)
 	flags = srl_last_flags;
 	srl_last_event = NULL;
 
+	srl_log_gnopernicus_real (event);
+
 	for (i = 0; i < SRL_MAX_CLIENTS; i++)
 	{
 	    if (srl_clients[i].event_proc)
@@ -330,6 +458,7 @@ notify_clients_ (gpointer data)
 		srl_clients[i].event_proc (event, flags);
 	    }	 
 	}
+	sre_release_reference (event);
     }
     busy = FALSE;
 
@@ -374,6 +503,7 @@ notify_clients (SREvent *event,
 	else
 	    g_idle_add (notify_clients_, NULL);
     }
+    srl_log_gnopernicus (event);
     srl_last_event = event;
     sre_add_reference (srl_last_event);
     srl_last_flags = flags;
@@ -477,6 +607,7 @@ srl_report_accessible_sro (Accessible *acc,
 	    ev->data = obj;
 	    sro_add_reference (obj);
 	    ev->data_destructor = (SREventDataDestructor) sro_release_reference;
+	    srl_log_important (NULL, ev);
 	    notify_clients (ev, 0);
 	    sre_release_reference (ev);
 	}
@@ -522,6 +653,7 @@ srl_process_event_real_window (SRLEvent *event)
 	    ev->data = obj;
 	    sro_add_reference (obj);
 	    ev->data_destructor = (SREventDataDestructor) sro_release_reference;
+	    srl_log_important (event, ev);
 	    notify_clients (ev, 0);
 	    sre_release_reference (ev);
 	}
@@ -550,6 +682,7 @@ srl_process_event_real_mouse (SRLEvent *event)
 	    point->x = event->detail1;
 	    point->y = event->detail2;
 	    ev->data_destructor = (SREventDataDestructor) g_free;
+	    srl_log_important (event, ev);
 	    notify_clients (ev, strcmp (event->type, "mouse:rel") ? 0 : 1);
 	}
 	sre_release_reference (ev);
@@ -575,6 +708,7 @@ srl_process_event_real_tooltip (SRLEvent *event)
 	    ev->data = obj;
 	    sro_add_reference (obj);
 	    ev->data_destructor = (SREventDataDestructor) sro_release_reference;
+	    srl_log_important (event, ev);
 	    notify_clients (ev, 0);
 	    sre_release_reference (ev);
 	}
@@ -590,6 +724,7 @@ srl_process_event_real_sro (SRLEvent *ev)
 {
     srl_assert (ev);
 
+    srl_log_important (ev, NULL);
     if (srl_last_delete_acc && 
 	Accessible_getRole (srl_last_delete_acc) == SPI_ROLE_COMBO_BOX &&
 	strcmp (ev->type, "object:property-changed:accessible-content") == 0)
@@ -1102,14 +1237,16 @@ report_event (const AccessibleEvent *event,
 	queue = g_queue_new ();
 
     g_queue_push_head (queue, ev);
+    srl_log_at_spi (event);
 
     if (busy)
     {
+	srl_log_reentrancy (event);
 	return;
     }
     busy = TRUE;
     
-    while (!g_queue_is_empty (queue))
+    while (queue && !g_queue_is_empty (queue))
     {
 	SRLEvent *ev1;
 	ev1 = (SRLEvent *) g_queue_pop_tail (queue);
@@ -1122,7 +1259,8 @@ report_event (const AccessibleEvent *event,
 	g_free (ev1);
     }
 
-    g_queue_free (queue);
+    if (queue)
+	g_queue_free (queue);
     queue = NULL;
     busy = FALSE;
 }
